from PyQt5.QtWidgets import QLabel, QPushButton, QDialog, QFormLayout, QVBoxLayout, QLineEdit, QDialogButtonBox
from PyQt5.QtWidgets import QStatusBar
from zbxnotifier.modules.zabbix.zabbix import ZabbixConnection
from zbxnotifier.modules.settings import Settings
from PyQt5.QtCore import QRunnable, pyqtSlot, pyqtSignal, QObject


class StatusSignals(QObject):
    finished = pyqtSignal()
    error = pyqtSignal(tuple)
    result = pyqtSignal(object)


class StatusWorker(QRunnable):
    def __init__(self):
        super(StatusWorker, self).__init__()
        self.signals = StatusSignals()

    @pyqtSlot()
    def run(self):
        connection_status = ZabbixConnection.get_status_desc()
        self.signals.result.emit(connection_status)


class Statusbar(QStatusBar):
    def __init__(self):
        super().__init__()
        self.status_widget = QLabel("ZBX Status: " + ZabbixConnection.get_status_desc())
        self.settings_button = QPushButton("SETTINGS")
        self.settings_button.clicked.connect(self.settings_clicked)
        self.setup_settings_dialog()
        self.init_elements()

    def setup_settings_dialog(self):
        self.settings_dialog = QDialog()
        self.settings_dialog.setWindowTitle("Settings")

        dialog_layout = QVBoxLayout()

        self.qline_zbx_server = QLineEdit(Settings.config.get('ZabbixSettings', 'server'))
        self.qline_zbx_username = QLineEdit(Settings.config.get('ZabbixSettings', 'username'))
        form_layout = QFormLayout()
        form_layout.addRow('Zabbix Server URL', self.qline_zbx_server)
        form_layout.addRow('Zabbix Username', self.qline_zbx_username)


        buttons = QDialogButtonBox(QDialogButtonBox.Cancel | QDialogButtonBox.Ok )

        buttons.clicked.connect(self.settings_dialog.accept, )
        buttons.accepted.connect(self.settings_ok_clicked)

        dialog_layout.addLayout(form_layout)
        dialog_layout.addWidget(QLabel("Clickin 'OK' will re-initialize the connection!"))
        dialog_layout.addWidget(buttons)

        dialog_layout.addWidget(QLabel("The supplied username must exist in the Windows Credential store under the 'zabbix' service!"))

        self.settings_dialog.setLayout(dialog_layout)

    def settings_ok_clicked(self):
        Settings.config.set('ZabbixSettings', 'username', self.qline_zbx_username.text())
        Settings.config.set('ZabbixSettings', 'server', self.qline_zbx_server.text())
        Settings.save_running_config()
        ZabbixConnection.re_init()

    def settings_clicked(self):
        self.settings_dialog.show()

    def update_elements(self):
        self.status_widget.setText("ZBX Status: " + ZabbixConnection.get_status_desc())
        self.status_widget.update()

    def init_elements(self):
        self.addWidget(self.settings_button)
        self.addWidget(self.status_widget)









class Problem:
    def __init__(self, triggerid, clock):
        self.triggerid = triggerid
        self.clock = clock
        self.trigger = None
        self.event = None

    def __str__(self):
        return self.triggerid + " " + self.clock + str(self.trigger)

    def __repr__(self):
        return self.__str__()


class Trigger:
    def __init__(self, triggerid, description, severity, lastchange_timestamp):
        self.triggerid = triggerid
        self.description = description
        self.severity = severity
        self.lastchange_timestamp = lastchange_timestamp

        self.severity_desc = None
        self.set_priority()

    def set_priority(self):
        if self.severity == '0':
            self.severity_desc = "Not Classified"
        elif self.severity == '1':
            self.severity_desc = "Information"
        elif self.severity == '2':
            self.severity_desc = "Warning"
        elif self.severity == '3':
            self.severity_desc = "Average"
        elif self.severity == '4':
            self.severity_desc = "High"
        elif self.severity == '5':
            self.severity_desc = "Critical"
        else:
            self.severity_desc = "UNKNOWN"

    def __str__(self):
        return self.severity + " " + self.description


class Host:
    def __init__(self, hostname, hostid):
        self.hostname = hostname
        self.hostid = hostid

    def __str__(self):
        return "Hostname: " + self.hostname + " " + self.hostid

    def __repr__(self):
        return self.__str__()

class Event:
    def __init__(self, triggerid, eventid, hosts):
        self.eventid = eventid
        self.triggerid = triggerid
        self.hosts = hosts





from pyzabbix.api import ZabbixAPI
from zbxnotifier.modules.settings import Settings
from zbxnotifier.modules.zabbix.elements import Trigger, Problem, Event, Host
from pyzabbix.api import ZabbixAPIException


class ZabbixConnection:
    connection = None
    token = None
    error = False
    error_message = ""

    @staticmethod
    def init():
        if ZabbixConnection.connection is None:
            try:
                ZabbixConnection.error = True
                ZabbixConnection.error_message = "Connecting..."
                ZabbixConnection.connection = ZabbixAPI(url=Settings.config.get('ZabbixSettings', 'server'), user=Settings.config.get('ZabbixSettings', 'username'), password=Settings.config.get('ZabbixSettings', 'password'))
                ZabbixConnection.token = ZabbixConnection.connection.auth
                ZabbixConnection.error_message = ""
                ZabbixConnection.error = False
            except ZabbixAPIException as e:
                ZabbixConnection.error = True
                ZabbixConnection.error_message = e.data

    @staticmethod
    def re_init():
        ZabbixConnection.connection = None
        ZabbixConnection.token = None
        ZabbixConnection.error = True
        ZabbixConnection.error_message = "Connection re-initializing"
        ZabbixConnection.init()

    @staticmethod
    def get_status_desc():
        if ZabbixConnection.connection is not None and ZabbixConnection.token is not None and ZabbixConnection.error is False:
            return "Connected"
        elif ZabbixConnection.connection is None and ZabbixConnection.token is None and ZabbixConnection.error is False:
            return "Logging in"
        elif ZabbixConnection.connection is None and ZabbixConnection.token is None and ZabbixConnection.error is True:
            return "Error: " + str(ZabbixConnection.error_message)
        return "Disconnected"

    @staticmethod
    def is_connected():
        if ZabbixConnection.connection is None:
            return False
        return True


class Zabbix:

    def __init__(self):
        ZabbixConnection.init()

    def get_problems(self):
        """
        Problems give object ids.
        If object is X, object id is:
            X=0: trigger
            X=4: item
            X=5 LLD rule
        :return: object_id list
        """
        data = ZabbixConnection.connection.problem.get(output='extend', selectAcknowledges="extend", selectTags="extend", selectSuppressionData="extend")
        problems = []
        for problem in data:
            problems.append(Problem(problem.get('objectid'), problem.get('clock')))
        return problems

    def get_events(self, trigger_ids):
        """
        Gets and event based on the event id
        objectid is always a trigger
        :param event_id:
        :return:
        """
        data = ZabbixConnection.connection.event.get(output="extend", objectids=trigger_ids, selectHosts="extend")
        events = []
        for event in data:
            hosts = []
            for host in event.get('hosts'):
                hosts.append(Host(host.get('name'), host.get('hostid')))
            events.append(Event(event.get('objectid'), event.get('eventid'), hosts))

        return events

    def get_triggers(self, trigger_ids):
        """
        Returns a trigger based on the trigger_id
        :param trigger_id:
        :return:
        """
        data = ZabbixConnection.connection.trigger.get(output="extend", triggerids=trigger_ids, expandDescription=True, status=0)
        triggers = []
        for trigger in data:
            triggers.append(Trigger(trigger.get('triggerid'), trigger.get('description'), trigger.get('priority'), trigger.get('lastchange')))

        return triggers







from PyQt5.QtGui import *
from PyQt5.QtWidgets import *


class ZbxProblemTable(QTableWidget):
    def __init__(self):
        super().__init__()

        self.setWindowTitle("Current Zabbix Alerts")
        self.resize(400, 250)

        self.problems = []
        self.init_table()

    def init_table(self):
        """
        Initial setup of the table. Sets the column headers.
        :return:
        """
        self.setColumnCount(5)
        self.setHorizontalHeaderLabels(['Time', 'Severity', 'Status', 'Host', 'Problem', 'Duration'])

        header = self.horizontalHeader()
        header.setSectionResizeMode(0, QHeaderView.ResizeToContents)
        header.setSectionResizeMode(1, QHeaderView.ResizeToContents)
        header.setSectionResizeMode(2, QHeaderView.ResizeToContents)
        header.setSectionResizeMode(3, QHeaderView.ResizeToContents)
        header.setSectionResizeMode(4, QHeaderView.Stretch)
        header.setSectionResizeMode(5, QHeaderView.Stretch)

        self._set_rows()

    def _get_bg_color(self, severity):
        if severity == '1':
            return [151, 170, 179]
        elif severity == '2':
            return [116, 153, 255]
        elif severity == '3':
            return [255, 200, 89]
        elif severity == '4':
            return [255, 160, 89]
        elif severity == '5':
            return [233, 118, 89]
        elif severity == '6':
            return [228, 89, 89]

    def update_data(self, problems):
        self.problems = problems
        self._set_rows()
        self._refresh_data()

    def _refresh_data(self):
        row = 0
        for problem in self.problems:
            severity = problem.trigger.severity
            hostnames = []
            for host in problem.event.hosts:
                hostnames.append(host.hostname)

            severity = QTableWidgetItem(severity)
            severity.setBackground(QColor(*self._get_bg_color(problem.trigger.severity)))

            hostname = QTableWidgetItem(', '.join(hostnames))
            problem_clock = QTableWidgetItem(problem.clock)
            trigger_description = QTableWidgetItem(problem.trigger.description)

            self.setItem(row, 0, problem_clock)
            self.setItem(row, 1, severity)
            self.setItem(row, 3, hostname)
            self.setItem(row, 4, trigger_description)
            row = row + 1
        self.update()
        self.show()

    def _set_rows(self):
        self.setRowCount(len(self.problems)+1)




import sys
from zbxnotifier.modules.windowelements.main_window import MainWindow
from zbxnotifier.modules.settings import Settings
from PyQt5.QtWidgets import QApplication


class Application:
    def __init__(self):
        app = QApplication(sys.argv)

        Settings.init_config()

        window = MainWindow()
        window.show()

        sys.exit(app.exec())




from zbxnotifier.modules.settings import Settings
from zbxnotifier.modules.windowelements.zbx_problem_table import ZbxProblemTable
from zbxnotifier.modules.zabbix.problems import ProblemsWorker
from zbxnotifier.modules.windowelements.statusbar import StatusWorker
from zbxnotifier.modules.windowelements.statusbar import Statusbar

from PyQt5.QtWidgets import QMainWindow
from PyQt5.QtCore import QTimer, QThreadPool


class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.setWindowTitle(Settings.config.get('WindowSettings', 'title'))
        self.window().resize(int(Settings.config.get('WindowSettings', 'width')), int(Settings.config.get('WindowSettings', 'height')))

        self.init_ui()

        self.problem_table_timer = QTimer()
        self.problem_table_timer.setInterval(1000)
        self.problem_table_timer.timeout.connect(self.update_problem_table_worker)
        self.problem_table_timer.start()

        self.statusbar_timer = QTimer()
        self.statusbar_timer.setInterval(500)
        self.statusbar_timer.timeout.connect(self.update_bar_worker)
        self.statusbar_timer.start()

    def init_ui(self):
        self._create_central_widget()
        self._create_statusbar()

        self.threadpool = QThreadPool()
        print("Multithreading with maximum %d threads" % self.threadpool.maxThreadCount())

    def _create_central_widget(self):
        """
        Initial setup of the widgets
        :return:
        """
        self.zbx_problem_table = ZbxProblemTable()
        self.setCentralWidget(self.zbx_problem_table)

    def _create_statusbar(self):
        self.statusbar = Statusbar()
        self.setStatusBar(self.statusbar)

    def update_bar_worker(self):
        worker = StatusWorker()
        worker.signals.result.connect(self.update_bar)

        self.threadpool.start(worker)

    def update_bar(self):
        self.statusbar.update_elements()

    def update_problem_table_worker(self):
        worker = ProblemsWorker()
        worker.signals.result.connect(self.update_problem_table)

        self.threadpool.start(worker)

    def update_problem_table(self, data):
        self.zbx_problem_table.update_data(data)
        self.update()

        self.size()











from PyQt5.QtCore import QRunnable, pyqtSlot, pyqtSignal, QObject
from zbxnotifier.modules.zabbix.zabbix import Zabbix, ZabbixConnection
import sys


class WorkerSignals(QObject):
    finished = pyqtSignal()
    error = pyqtSignal(tuple)
    result = pyqtSignal(object)


class ProblemsWorker(QRunnable):
    def __init__(self):
        super(ProblemsWorker, self).__init__()
        self.signals = WorkerSignals()
        self.zbx = Zabbix()

    @pyqtSlot()
    def run(self):
        try:
            problems = self.get_problems()
            self.signals.result.emit(problems)
        except:
            (type, value, traceback) = sys.exc_info()
            print(type)
            print(value)

    def get_problems(self):
        # Get all the problems
        if not ZabbixConnection.is_connected():
            print("We are not connected to the Zabbix server.")
            return []

        problems = self.zbx.get_problems()

        # Get all the triggers
        trigger_ids = []
        for problem in problems:
            trigger_ids.append(problem.triggerid)
        triggers = self.zbx.get_triggers(trigger_ids)

        for problem in problems:
            for trigger in triggers:
                # Sometimes, there's no trigger id for the problemid
                try:
                    if problem.triggerid == trigger.triggerid:
                        problem.trigger = trigger
                except AttributeError:
                    problem.trigger = None

        # Get all events based on the trigger IDs
        events = self.zbx.get_events(trigger_ids)

        for problem in problems:
            for event in events:
                try:
                    if problem.trigger.triggerid == event.triggerid:
                        problem.event = event
                except AttributeError:
                    problem.event = None

        # Search for faulty problems, when there's no trigger or event
        clear_problems = []
        for problem in problems:
            if problem.trigger is not None and problem.event is not None:
                clear_problems.append(problem)

        return clear_problems



from distutils.core import setup


setup(
  name = 'zbxnotifier',
  packages = ['zbxnotifier', 'zbxnotifier/modules', 'zbxnotifier/modules/windowelements', 'zbxnotifier/modules/zabbix'],
  version = '0.5.1',
  license='gpl-3.0',
  description = 'Simple Zabbix Notifier Desktop Application',
  author = 'Daniel Kovacs',
  author_email = 'kovacsdanielhun@gmail.com',
  url = 'https://github.com/inframates/zbxnotifier',
  download_url = 'https://github.com/inframates/zbxnotifier/archive/0.5.1.tar.gz',
  keywords = ['Zabbix', 'desktop', 'monitoring'],
  install_requires=[
        'PyQt5',
        'py-zabbix',
        'keyring<=18',
        'appdirs',
        'configparser'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',
    'Topic :: System :: Monitoring',
    'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    'Programming Language :: Python :: 3',
  ],
)


import sys

if sys.platform == 'darwin':
    import keyring.backends.OS_X
    keyring.set_keyring(keyring.backends.OS_X.Keyring())
else:
    import keyring.backends.Windows
    keyring.set_keyring(keyring.backends.Windows.WinVaultKeyring())

from appdirs import *
import configparser
import os.path


class Settings:
    config = None
    appname = "zbxnotifier"
    appauthor = "kovaxur"

    user_data_dir = None
    user_data_config_path = None

    @staticmethod
    def init_config():
        Settings.user_data_dir = user_data_dir(Settings.appname, Settings.appauthor)
        Settings.user_data_config_path = Settings.user_data_dir + '/zbxnotifier.ini'

        if not os.path.exists(Settings.user_data_config_path):
            Settings.create_default_config()
            Settings.save_running_config()
        else:
            Settings.read_config_from_file()

    @staticmethod
    def create_default_config():
        Settings.config = configparser.ConfigParser()

        Settings.config.add_section('WindowSettings')
        Settings.config.set('WindowSettings', 'title', "RoboAlert 3000")
        Settings.config.set('WindowSettings', 'height', str(500))
        Settings.config.set('WindowSettings', 'width', str(600))

        Settings.config.add_section('ZabbixSettings')
        Settings.config.set('ZabbixSettings', 'username', 'Admin')
        Settings.config.set('ZabbixSettings', 'server', 'http://192.168.1.160')

        password = keyring.get_password("zabbix", "Admin")
        if password is None:
            print("Password must be set in the Windows Credential store.")
            password = ''
        Settings.config.set('ZabbixSettings', 'password', password)

        try:
            os.makedirs(Settings.user_data_dir)
        except FileExistsError:
            pass

    @staticmethod
    def save_running_config():
        # We dont want to print the password into the settings file..
        temp_pw = Settings.config.get('ZabbixSettings', 'password')
        Settings.config.set('ZabbixSettings', 'password', 'XXXXXXXXXX')

        with open(Settings.user_data_config_path, 'w') as configfile:
            Settings.config.write(configfile)

        Settings.config.set('ZabbixSettings', 'password', temp_pw)

    @staticmethod
    def read_config_from_file():
        Settings.config = configparser.ConfigParser()
        Settings.config.read(Settings.user_data_dir + '/zbxnotifier.ini')
        Settings.config.set('ZabbixSettings', 'password', keyring.get_password("zabbix", "Admin"))